from PyQt5.QtXml import QDomDocument
from qgis.core import QgsReadWriteContext, QgsLayoutItemMap, \
    QgsLayoutItemLabel, QgsLayoutItemPicture, QgsLayoutItemScaleBar
from . import paggf_version
import sip


class DBLayoutStorage:
    def __init__(self, storing_field_name='xml_parcel_sketch'):
        self.items_to_store = {}  # dict of item_id and item class
        # self.layer = layer
        self.layer = None
        self.page = None
        self.storing_field_name = storing_field_name

    def store(self, feature, layer, layout):
        doc = QDomDocument()
        root = doc.createElement("data")
        doc.appendChild(root)
        context = QgsReadWriteContext()

        for item_id in self.items_to_store:
            item_raw = layout.itemById(item_id)
            if item_raw:
                item = sip.cast(item_raw, self.items_to_store[item_id])
                doc_elem = doc.createElement(item_id)

                if item.writeXml(doc_elem, doc, context):
                    root.appendChild(doc_elem)

        xml = doc.toString()

        layer.startEditing()
        layer.changeAttributeValue(
            feature.id(),
            layer.fields().indexFromName(self.storing_field_name), xml)

    def load(self, feature, layout):
        xml = feature[self.storing_field_name]

        if xml:
            doc = QDomDocument()
            doc.setContent(xml)
            context = QgsReadWriteContext()

            root = doc.firstChildElement("data")

            for item_id in self.items_to_store:
                item_raw = layout.itemById(item_id)
                if item_raw:
                    elem_xml_container = root.firstChildElement(item_id)
                    elem_xml = elem_xml_container.firstChildElement(
                        'LayoutItem')
                    elem = sip.cast(item_raw, self.items_to_store[item_id])
                    if elem.readXml(elem_xml, doc, context):
                        if self.page:
                            elem.attemptMove(elem.positionWithUnits(), page=self.page)
                    else:
                        print('{} : pbm'.format(item_id))
        else:
            print('no xml to load')


class Loader:
    ret = {}

    @classmethod
    def get(cls, version):
        if version not in cls.ret:
            items_to_store = {}

            if version == paggf_version.BM:
                items_to_store = {
                    'carto': QgsLayoutItemMap,
                    'cartoText': QgsLayoutItemLabel,
                    'cartoProjection': QgsLayoutItemLabel,
                    'cartoScalebar': QgsLayoutItemScaleBar,
                    'cartoNorthArrow': QgsLayoutItemPicture,
                }

            elif version == paggf_version.CID:
                items_to_store = {
                    'carto': QgsLayoutItemMap,
                    'cartoTextCID': QgsLayoutItemLabel,
                    'cartoText': QgsLayoutItemLabel,
                    'cartoProjection': QgsLayoutItemLabel,
                    'cartoScalebar': QgsLayoutItemScaleBar,
                    'cartoScalebarContainer': QgsLayoutItemLabel,
                    'cartoNorthArrow': QgsLayoutItemPicture,
                }

            cls.ret[version] = DBLayoutStorage()
            cls.ret[version].items_to_store = items_to_store

        return cls.ret[version]
