from PyQt5 import uic
from .paggf_dlg import PAGGFDialog
from qgis.gui import QgsAttributeDialog
from PyQt5.QtCore import QLocale
from .ALL_generate_report import EFGenerateReportDlg, TFGenerateReportDlg
import os.path


class EFTFDlg(PAGGFDialog):
    def __init__(self, iface, parent=None):
        super(EFTFDlg, self).__init__()
        self.iface = iface
        self.setupUi(self)

        self.fr_locale = QLocale(QLocale.French, QLocale.France)

    def ef_encode_slot(self):
        self.ef_mode = True
        self.working_layer_name = 'ef_parcels_to_valid'
        self.select_and_display_form()

    def tf_encode_slot(self):
        self.ef_mode = False
        self.working_layer_name = 'tf_parcels_to_valid'
        self.select_and_display_form()

    def select_and_display_form(self):
        self.working_layer = self.layers.get('gis', self.working_layer_name)
        layer_for_selection = self.working_layer
        if layer_for_selection.selectedFeatureCount() != 1:
            if self.ef_mode:
                layer_for_selection = self.layers.get(
                    'gis', 'tf_parcels_to_valid')
            else:
                layer_for_selection = self.layers.get(
                    'gis', 'ef_parcels_to_valid')

        if layer_for_selection.selectedFeatureCount() != 1:
            self.iface.messageBar().pushCritical(
                'Erreur',
                'Veuillez sélectionner une entité sur la couche "{}" pour\
cette action'.format(self.working_layer.name()))
        else:
            feature = layer_for_selection.selectedFeatures()[0]
            if layer_for_selection != self.working_layer:
                feature = self.layers.get_feature(
                    'gis', self.working_layer_name, feature['idc_parcel_code'],
                    'idc_parcel_code')
            self.working_layer.startEditing()
            self.working_feature = feature

            self.callback()


FORM_CLASS_GENERATE, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'EFTF_generate_print.ui'))


class EFTFPrintDlg(EFTFDlg, FORM_CLASS_GENERATE):
    def __init__(self, iface, parent=None):
        super(EFTFPrintDlg, self).__init__(iface)

        self.ef_generate.clicked.connect(self.ef_encode_slot)
        self.tf_generate.clicked.connect(self.tf_encode_slot)

    def callback(self):
        if self.ef_mode:
            self.dlg = EFGenerateReportDlg(
                self.iface, self.working_layer, self.working_feature)
        else:
            self.dlg = TFGenerateReportDlg(
                self.iface, self.working_layer, self.working_feature)
        self.close()
        self.dlg.exec_()


FORM_CLASS_ENCODE, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'EFTF_encode.ui'))


class EFTFEncodeDlg(EFTFDlg, FORM_CLASS_ENCODE):
    def __init__(self, iface, parent=None):
        super(EFTFEncodeDlg, self).__init__(iface)

        self.ef_encode.clicked.connect(self.ef_encode_slot)
        self.tf_encode.clicked.connect(self.tf_encode_slot)

    def closing_attribute_dialog_slot(self):
        self.deleteLater()

    def callback(self):
        self.attribute_dialog = QgsAttributeDialog(
            self.working_layer, self.working_feature, True,
            parent=self.iface.mainWindow())
        self.attribute_dialog.show()
        self.attribute_dialog.finished.connect(
            self.closing_attribute_dialog_slot)
        self.close()
